@@ -21,6 +21,7 @@
 
 void init_builtin_modules(dict_t *dict)
 {
+    pubkeys_init(dict);
     resolver_module(dict);
     random_module(dict);
     udp_module(dict);
@@ -37,4 +38,10 @@ void init_builtin_modules(dict_t *dict)
     tun_module(dict);
     sha1_module(dict);
     log_module(dict);
+    privcache_module(dict);
 }
+
+const struct sigscheme_info sigschemes[]={
+    { "rsa1", 0x00, rsa1_loadpub, rsa1_loadpriv },
+    { 0 }
+};
